from ftw.builder import builder_registry
from ftw.builder.dexterity import DexterityBuilder
from plone.namedfile.file import NamedImage
from StringIO import StringIO


class SliderContainerBuilder(DexterityBuilder):
    portal_type = 'ftw.slider.Container'

builder_registry.register('slider container', SliderContainerBuilder)


class SliderPaneBuilder(DexterityBuilder):
    portal_type = 'ftw.slider.Pane'

    def with_dummy_image(self):
        dummy_image = StringIO(
            'GIF89a\x01\x00\x01\x00\x80\x00\x00\x00\x00\x00\x00\x00'
            '\x00!\xf9\x04\x04\x00\x00\x00\x00,\x00\x00\x00\x00\x01\x00'
            '\x01\x00\x00\x02\x02D\x01\x00;')

        self.arguments["image"] = NamedImage(dummy_image.read(),
                                                 filename=u"image.gif")
        return self

builder_registry.register('slider pane', SliderPaneBuilder)



from ftw.builder import Builder
from ftw.builder import create
from ftw.slider.testing import SLIDER_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.textfield.value import RichTextValue
from unittest2 import TestCase


class TestSliderCreation(TestCase):

    layer = SLIDER_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        setRoles(self.portal, TEST_USER_ID, ['Contributor', 'Manager'])
        login(self.portal, TEST_USER_NAME)

        self.folder = create(Builder('folder')
                             .titled(u'Folder'))

        self.container = create(Builder('slider container')
                                .within(self.folder)
                                .titled(u'Slider Container'))

        create(Builder('slider pane')
               .within(self.container)
               .titled(u'Pane 1')
               .with_dummy_image())

    @browsing
    def test_slider_is_visible_on_folder(self, browser):
        browser.login().visit(self.folder)
        self.assertEqual(1, len(browser.css('#slider-wrapper')))

    @browsing
    def test_slider_is_invisible_on_page(self, browser):

        page = create(Builder('page')
                      .within(self.folder)
                      .titled(u'Page'))

        browser.login().visit(page)
        self.assertEqual(0, len(browser.css('#slider-wrapper')))

    @browsing
    def test_slider_is_visible_on_default_page(self, browser):

        create(Builder('page')
               .within(self.folder)
               .titled(u'Page'))

        # set default page
        self.folder.setDefaultPage('page')
        import transaction
        transaction.commit()

        browser.login().visit(self.folder)

        self.assertEqual(1, len(browser.css('body.portaltype-document')))
        self.assertEqual(1, len(browser.css('#slider-wrapper')))

    @browsing
    def test_slider_has_special_css_class_if_text_is_available(self, browser):
        browser.login().visit(self.folder)
        self.assertEqual(0, len(browser.css('sliderPaneHasText')))

        create(Builder('slider pane')
               .within(self.container)
               .titled(u'Pane 2')
               .having(text=RichTextValue("Test text"))
               .with_dummy_image())

        browser.visit(self.folder)

        self.assertEqual(1, len(browser.css('.sliderPaneHasText')))



from ftw.upgrade import UpgradeStep


class SetShowTitleOnExistingSliderPanes(UpgradeStep):
    """Set "show_title" on existing slider panes.
    """

    def __call__(self):
        self.install_upgrade_profile()
        slider_panes = self.objects({'portal_type': 'ftw.slider.Pane'},
                                    message='Set "show_title" of existing slider panes.')
        for slider_pane in slider_panes:
            if slider_pane.text:
                slider_pane.show_title = True



from ftw.upgrade import UpgradeStep


class RemoveIBasicBehaviorFromPane(UpgradeStep):
    """Remove i basic behavior from pane.
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.testing import IS_PLONE_5
from plone import api
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
import transaction


class LanguageToolHelper(object):
    def __init__(self):
        self.lang_tool = None

    def manage_set_language_settings(self,
                                     default='en',
                                     supported=None,
                                     use_combined=False,
                                     start_neutral=True,
                                     content_negotiation=True):
        """
        Sets language settings regardeless if plone4.3 or plone5.1
        :param default: default site language
        :param supported: list of supported languages
        :param use_combined: setUseCombinedLanguageCodes in Plone 4.x
        :param start_neutral: startNeutral in Plone 4.x
        :param content_negotiation: default is False
        """
        # startNeutral is not used/available in plone 5.1 anymore

        if not supported:
            supported = ['en']

        if IS_PLONE_5:
            from Products.CMFPlone.interfaces import ILanguageSchema

            self.lang_tool = api.portal.get_tool('portal_languages')
            self.lang_tool.setDefaultLanguage(default)
            for lang in supported:
                self.lang_tool.addSupportedLanguage(lang)
            self.lang_tool.settings.use_combined_language_codes = False
            self.lang_tool.setLanguageCookie()
            registry = getUtility(IRegistry)
            language_settings = registry.forInterface(ILanguageSchema,
                                                      prefix='plone')
            language_settings.use_content_negotiation = content_negotiation
            language_settings.default_language = default

        else:
            self.lang_tool = self.portal.portal_languages
            self.lang_tool.manage_setLanguageSettings(
                default,
                supported,
                setUseCombinedLanguageCodes=use_combined,
                # Set this only for better testing ability
                setCookieEverywhere=True,
                startNeutral=start_neutral,
                setContentN=True)
        transaction.commit()



from ftw.referencewidget.widget import ReferenceWidgetFactory
from ftw.slider import _
from ftw.slider.interfaces import IPane
from pkg_resources import get_distribution
from plone.app.textfield import RichText
from plone.dexterity.content import Container
from plone.directives import form
from plone.directives.form import Schema
from plone.namedfile.field import NamedImage
from plone.supermodel import model
from zope import schema
from zope.interface import Invalid
from zope.interface import implements
from zope.interface.interface import invariant


IS_PLONE_5 = get_distribution('Plone').version >= '5'

if not IS_PLONE_5:
    from plone.app.dexterity import PloneMessageFactory as _PMF
else:
    from Products.CMFPlone import PloneMessageFactory as _PMF


class IPaneSchema(Schema):

    title = schema.TextLine(
        title=_PMF(u'label_title', default=u'Title'),
        required=True
        )

    show_title = schema.Bool(
        title=_(u'label_show_title', default=u'Show title'),
        required=False,
        default=False,
        missing_value=False,
    )

    text = RichText(
        title=_(u'label_text', default=u'Text'),
        description=_(u'help_text', default=u''),
        required=False,
        )

    model.primary('image')
    image = NamedImage(
        title=_(u'label_image', default='Image'),
        required=True,
        )

    image_alt_text = schema.TextLine(
        title=_(u'label_image_alt_text', default=u'Image alt text'),
        required=False
        )

    form.widget(link=ReferenceWidgetFactory)
    link = schema.TextLine(
        title=_(u'label_link', default=u'Link'),
        description=_(u'help_link', default=u''),
        required=False,
        )

    external_url = schema.URI(
        title=_(u'label_external_url', default=u'External URL'),
        required=False,
    )

    @invariant
    def validate_links(data):
        """
        It is not allowed to add a link and an external url at the same time.
        """
        if data.link and data.external_url:
            raise Invalid(_(
                u'error_message_links',
                default=u'It\'s not possible to add a link and an '
                        u'external url at the same time')
            )


class Pane(Container):
    implements(IPane)



from ftw.slider import _
from ftw.slider.browser import utils
from plone.dexterity.browser.add import DefaultAddForm, DefaultAddView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from zope.i18n import translate
from zope.publisher.browser import BrowserView
import json


class SliderView(BrowserView):

    template = ViewPageTemplateFile('slider.pt')

    def __call__(self):
        return self.template()

    def panes(self):
        return self.context.getFolderContents(full_objects=True)

    def extend_translations(self, config):
        labels = {
            'play': _(u'label_slider_play', default=u'Play'),
            'pause': _(u'label_slider_pause', default=u'Pause'),
            'next': _(u'label_slider_next', default=u'Next'),
            'prev': _(u'label_slider_prev', default=u'Previous'),
        }

        user_config = config.get('labels', {})

        for label, message in labels.iteritems():
            if label not in user_config:
                user_config[label] = translate(message, context=self.request)

        config['labels'] = user_config

        return config

    def get_slick_config(self):
        # The config value may contain unwanted new lines. Let's remove them
        # by loading and dumping as json.
        if not self.context.slick_config:
            return '{}'
        config = json.loads(self.context.slick_config)
        config = self.extend_translations(config)
        return json.dumps(config)

    def get_pane_link(self, pane):
        return utils.get_pane_link(pane)


class ContainerAddForm(DefaultAddForm):

    def render(self):
        sliders = self.context.getFolderContents(
            contentFilter={'portal_type': 'ftw.slider.Container'})

        if len(sliders) > 0:
            IStatusMessage(self.request).add(
                _(u'There is already a slider pane in this context'),
                type='info')
            return self.request.response.redirect(sliders[0].getURL())
        return super(ContainerAddForm, self).render()


class ContainerAddView(DefaultAddView):
    form = ContainerAddForm



from zope.interface import Interface


SLIDER_VIEW = 'slider_view'


class IContainer(Interface):
    """Marker interface for the slider container.
    """


class IPane(Interface):
    """Marker interface for the slider pane.
    """



from ftw.builder import Builder
from ftw.builder import create
from ftw.slider.testing import SLIDER_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from ftw.testbrowser.pages import plone
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest2 import TestCase
import transaction


class TestSliderCreation(TestCase):
    layer = SLIDER_FUNCTIONAL_TESTING

    def setUp(self):
        setRoles(self.layer['portal'], TEST_USER_ID, ['Contributor', 'Manager'])
        transaction.commit()

    @browsing
    def test_slider_container_is_addable(self, browser):
        browser.login().open()
        factoriesmenu.add('Slider Container')
        browser.fill({'Title': 'Container 1'}).submit()
        self.assertEquals('Container 1', plone.first_heading())
        self.assertEquals('ftw-slider-container', plone.portal_type())

    @browsing
    def test_redirect_if_there_is_already_a_container(self, browser):
        container = create(Builder('slider container').titled(u'Slider Container'))
        browser.login().open()
        factoriesmenu.add('Slider Container')
        self.assertEquals(container.absolute_url(), browser.url)



from ftw.upgrade import UpgradeStep


class SetEffectiveAndExpiryDateOnSliderContainerAndPanes(UpgradeStep):
    """Set effective and expiry date on slider container and panes.
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.slider.browser import utils
from plone import api
from Products.Five.browser import BrowserView


class SliderPaneView(BrowserView):

    def __call__(self):
        redirect_url = self.get_redirect_url()
        if redirect_url:
            return self.request.RESPONSE.redirect(redirect_url)
        else:
            return super(SliderPaneView, self).__call__()

    def get_redirect_url(self):
        """Redirect view adopted from plone's link_redirect_view.py"""

        can_edit = api.user.has_permission('Modify portal content',
                                           obj=self.context)

        link = self.get_link()
        if not can_edit and link:
            return link
        else:
            return ''

    def get_link(self):
        return utils.get_pane_link(self.context)



from Acquisition import aq_parent, aq_inner
from ftw.slider.interfaces import SLIDER_VIEW
from plone.app.layout.viewlets import ViewletBase
from zope.component import getMultiAdapter


class SliderViewlet(ViewletBase):

    def render(self):
        context = self.context

        plone = getMultiAdapter((self.context, self.request), name="plone")
        if plone.isDefaultPageInFolder():
            context = aq_parent(aq_inner(self.context))

        containers = context.getFolderContents(
            contentFilter={'portal_type': 'ftw.slider.Container'},
            full_objects=True)
        self.container = len(containers) > 0 and containers[0] or None
        if self.container:
            view = self.container.restrictedTraverse(SLIDER_VIEW, None)
            if view:
                return view()
        return ''



import transaction
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from unittest2 import TestCase

from ftw.slider.testing import SLIDER_FUNCTIONAL_TESTING


class FunctionalTestCase(TestCase):
    layer = SLIDER_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def grant(self, *roles):
        setRoles(self.portal, TEST_USER_ID, list(roles))
        transaction.commit()



from ftw.builder import Builder
from ftw.builder import create
from ftw.slider.tests import FunctionalTestCase
from ftw.testbrowser import browsing
from ftw.testbrowser.pages.statusmessages import assert_message
from plone.app.textfield import RichTextValue


class TestSliderPane(FunctionalTestCase):

    def setUp(self):
        super(TestSliderPane, self).setUp()
        self.grant('Manager')

    @browsing
    def test_slider_pane_cannot_have_link_and_external_url(self, browser):
        """
        This test makes sure that the slider pane cannot have a link
        and an external url at the same time.
        """
        folder = create(Builder('folder').titled(u'Folder'))

        container = create(Builder('slider container')
                           .within(folder)
                           .titled(u'Slider Container'))

        pane = create(Builder('slider pane')
                      .within(container)
                      .titled(u'Pane 1')
                      .with_dummy_image())

        my_folder = create(Builder('folder').titled(u'My Folder'))

        browser.login().visit(pane)
        browser.find('Edit').click()
        browser.fill({
            'Link': my_folder,
            'External URL': 'http://4teamwork.ch',
        })
        browser.find_button_by_label('Save').click()

        assert_message('There were some errors.')

    @browsing
    def test_slider_pane_without_text_does_not_render_any_content(self, browser):
        """
        This test makes sure that a slider pane without text does not render any
        content (neither the title nor the text).
        """
        container = create(Builder('slider container')
                           .titled(u'Slider Container'))
        create(Builder('slider pane')
               .titled(u'The title of the pane')
               .with_dummy_image()
               .within(container))

        browser.visit(container, view='slider_view')
        self.assertEqual(
            [''],
            browser.css('.sliderPane').text
        )

    @browsing
    def test_slider_pane_with_text_does_only_render_the_text(self, browser):
        """
        This test makes sure that a slider pane having some text will really
        render the text.
        """
        container = create(Builder('slider container')
                           .titled(u'Slider Container'))
        create(Builder('slider pane')
               .titled(u'The title of the pane')
               .having(text=RichTextValue('The text of the pane'))
               .with_dummy_image()
               .within(container))

        browser.visit(container, view='slider_view')
        self.assertEqual(
            ['The text of the pane'],
            browser.css('.sliderPane').text
        )

    @browsing
    def test_slider_pane_with_title_but_without_text(self, browser):
        """
        This test makes sure that a slider pane configured to show the title but
        without text will really render the title.
        """
        container = create(Builder('slider container')
                           .titled(u'Slider Container'))
        create(Builder('slider pane')
               .titled(u'The title of the pane')
               .having(show_title=True)
               .with_dummy_image()
               .within(container))

        browser.visit(container, view='slider_view')
        self.assertEqual(
            ['The title of the pane'],
            browser.css('.sliderPane').text
        )

    @browsing
    def test_slider_pane_with_title_and_with_text(self, browser):
        """
        This test makes sure that a slider pane configured to show the title
        and having some text will render both the title and the text.
        """
        container = create(Builder('slider container')
                           .titled(u'Slider Container'))
        create(Builder('slider pane')
               .titled(u'The title of the pane')
               .having(text=RichTextValue('The text of the pane'))
               .having(show_title=True)
               .with_dummy_image()
               .within(container))

        browser.visit(container, view='slider_view')
        self.assertEqual(
            ['The title of the pane The text of the pane'],
            browser.css('.sliderPane').text
        )



from ftw.builder import Builder
from ftw.builder import create
from ftw.slider.testing import SLIDER_FUNCTIONAL_TESTING
from ftw.testbrowser import browsing
from ftw.testbrowser.pages.statusmessages import info_messages
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from unittest2 import TestCase


class TestSliderPaneView(TestCase):

    layer = SLIDER_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Contributor', 'Manager'])
        login(self.portal, TEST_USER_NAME)

        self.folder = create(Builder('folder')
                             .titled(u'Folder'))

        self.container = create(Builder('slider container')
                                .within(self.folder)
                                .titled(u'Slider Container'))

    @browsing
    def test_do_not_redirect_if_there_is_no_link(self, browser):
        pane = create(Builder('slider pane')
                      .within(self.container)
                      .titled(u'Pane 1')
                      .with_dummy_image())

        browser.login().visit(pane)
        self.assertEquals(pane.absolute_url(), browser.url)

    @browsing
    def test_do_not_redirect_external_url_if_user_can_edit(self, browser):
        pane = create(Builder('slider pane')
                      .within(self.container)
                      .titled(u'Pane 1')
                      .having(external_url=self.portal.absolute_url())
                      .with_dummy_image())

        browser.login().visit(pane)
        self.assertEquals(pane.absolute_url(), browser.url)

    @browsing
    def test_do_not_redirect_internal_url_if_user_can_edit(self, browser):
        target = create(Builder('folder')
                        .titled(u'Target folder'))

        portal_path = '/'.join(self.portal.getPhysicalPath())
        pane = create(Builder('slider pane')
                      .within(self.container)
                      .titled(u'Pane 1')
                      .having(link='/'.join(target.getPhysicalPath())[len(portal_path):])
                      .with_dummy_image())

        browser.login().visit(pane)
        self.assertEquals(pane.absolute_url(), browser.url)

    @browsing
    def test_redirect_internal_url_if_user_cannot_edit(self, browser):
        target = create(Builder('folder')
                        .titled(u'Target folder'))

        portal_path = '/'.join(self.portal.getPhysicalPath())
        pane = create(Builder('slider pane')
                      .within(self.container)
                      .titled(u'Pane 1')
                      .having(link='/'.join(target.getPhysicalPath())[len(portal_path):])
                      .with_dummy_image())

        browser.visit(pane)
        self.assertEquals(target.absolute_url(), browser.url)

    @browsing
    def test_redirect_external_url_if_user_cannot_edit(self, browser):
        pane = create(Builder('slider pane')
                      .within(self.container)
                      .titled(u'Pane 1')
                      .having(external_url=self.portal.absolute_url())
                      .with_dummy_image())

        browser.visit(pane)
        self.assertEquals(self.portal.absolute_url(), browser.url)

    @browsing
    def test_internal_url_on_slider_view(self, browser):
        target = create(Builder('folder')
                        .titled(u'Target folder'))

        portal_path = '/'.join(self.portal.getPhysicalPath())
        create(Builder('slider pane')
               .within(self.container)
               .titled(u'Pane 1')
               .having(link='/'.join(target.getPhysicalPath())[len(portal_path):])
               .with_dummy_image())

        # The internal links of the panes in the slider view must be of the form
        # "http://nohost/plone/target-folder" and not only "/target-folder" in order
        # for the links to work on localhost too.
        browser.login().visit(self.container, view='slider_view')
        self.assertEqual(
            ['http://nohost/plone/target-folder'],
            [link.attrib['href'] for link in browser.css('.sliderPane a')]
        )



from ftw.upgrade import UpgradeStep


class MakeSliderPanesAddableOnSiteRootForSiteManager(UpgradeStep):
    """Make slider panes addable on site root for site manager.
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.testing import browser
from ftw.testing.pages import Plone


class ContentMenu(Plone):

    @property
    def content_views_links(self):
        return browser().find_by_css('#content-views a')

    @property
    def content_views_labels(self):
        return map(lambda link: link.text.strip(), self.content_views_links)

    @property
    def content_actions_labels(self):
        items = browser().find_by_css('#plone-contentmenu-actions dd a')
        return map(lambda link: link.text.strip(), items)

    @property
    def addable_types_links(self):
        nodes = browser().find_by_css('#plone-contentmenu-factories li a')
        return dict([(node.text.strip(), node) for node in nodes])

    @property
    def addable_types_labels(self):
        return self.addable_types_links.keys()



from ftw.upgrade import UpgradeStep


class InstallFtwReferencewidget(UpgradeStep):
    """Install ftw.referencewidget.
    """

    def __call__(self):

        self.setup_install_profile('profile-ftw.referencewidget:default')
        self.install_upgrade_profile()



from ftw.upgrade import UpgradeStep
from Products.CMFPlone.utils import getFSVersionTuple


class ProvidePrecompiledBundle(UpgradeStep):
    """Provide precompiled bundle.
    """

    def __call__(self):
        if getFSVersionTuple() > (5, ):
            self.install_upgrade_profile()



from setuptools import setup, find_packages
import os

version = '3.4.2'

tests_require = [
    'ftw.testing',
    'ftw.builder',
    'ftw.testbrowser',
    'plone.app.testing',
    'plone.resource',
    'unittest2',
]

setup(name='ftw.slider',
      version=version,
      description='Slider content type for Plone.',
      long_description=open('README.rst').read() + '\n' +
      open(os.path.join('docs', 'HISTORY.txt')).read(),

      # Get more strings from
      # http://pypi.python.org/pypi?:action=list_classifiers
      classifiers=[
          'Framework :: Plone :: 4.3',
          'Framework :: Plone :: 5.1',
          'Framework :: Plone',
          'Programming Language :: Python',
      ],
      keywords='',

      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      url='https://github.com/4teamwork/ftw.slider',
      license='GPL2',

      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw'],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
          'ftw.upgrade',
          'ftw.referencewidget',
          'setuptools',
          'Products.GenericSetup',
          'Products.CMFCore',
          'Products.CMFPlone',
          'plone.api',
          'plone.app.jquery >= 1.7.2',
          'plone.behavior',
          'plone.namedfile',
          'plone.dexterity',
          'plone.app.dexterity',
          'plone.directives.form',
          'archetypes.schemaextender',
      ],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),

      entry_points='''
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      ''',
      )



from plone import api


def get_pane_link(pane):
    link = pane.link
    if link:
        return '{0}{1}'.format(
            api.portal.get().absolute_url(),
            link.startswith('/') and link or '/' + link)

    elif pane.external_url:
        return pane.external_url

    else:
        return ''



from ftw.upgrade import UpgradeStep


class InstallContentTreeWidget(UpgradeStep):

    def __call__(self):
        profileid = 'plone.formwidget.contenttree:default'
        version = self.portal_setup.getLastVersionForProfile(profileid)
        if version == 'unknown':
            self.setup_install_profile('profile-' + profileid)



from ftw.slider import _
from ftw.slider.interfaces import IContainer
from plone.dexterity.content import Container
from plone.directives.form import Schema
from zope import schema
from zope.interface import implements
import json


def validate_slick_config(value):
    try:
        json.loads(value)
    except ValueError:
        return False

    return True


class IContainerSchema(Schema):
    slick_config = schema.Text(
        title=_(u'slick_config_label', default=u'Configuration'),
        description=_(u'slick_config_description',
                      default=u'See http://kenwheeler.github.io/slick/'),
        required=False,
        default=u'''
            {"autoplay": true,
             "autoplaySpeed": 5000,
             "fade": true,
             "speed": 1500,
             "infinite": true}
        ''',
        constraint=validate_slick_config,
    )


class Topic(Container):
    implements(IContainer)



from ftw.builder import Builder
from ftw.builder import create
from ftw.slider.contents.container import validate_slick_config
from ftw.slider.testing import SLIDER_FUNCTIONAL_TESTING
from ftw.slider.testing import SLIDER_INTEGRATION_TESTING
from ftw.testbrowser import browsing
from helper import LanguageToolHelper
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from unittest2 import TestCase
import json


class TestSliderConfig(TestCase, LanguageToolHelper):

    layer = SLIDER_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        setRoles(self.portal, TEST_USER_ID, ['Contributor', 'Manager'])
        login(self.portal, TEST_USER_NAME)

        self.folder = create(Builder('folder')
                             .titled(u'Folder'))

    @browsing
    def test_slider_adds_slick_config_if_available(self, browser):
        self.manage_set_language_settings(default='de',
                                          supported=['de'],
                                          use_combined=False,
                                          content_negotiation=False)

        container = create(Builder('slider container')
                           .within(self.folder)
                           .having(slick_config=json.dumps({'is_chuck': True}))
                           .titled(u'Slider Container'))

        create(Builder('slider pane')
               .within(container)
               .titled(u'Pane 1')
               .with_dummy_image())

        browser.login().visit(self.folder)

        self.assertEqual(
            {u'labels': {u'pause': u'Pause',
                         u'play': u'Abspielen',
                         u'prev': u'Vorheriges Bild',
                         u'next': u'N\xe4chstes Bild'},
             u'is_chuck': True},
            json.loads(browser.css('#slider-panes').first.get('data-settings')))


class TestSliderConfigValidator(TestCase):

    layer = SLIDER_INTEGRATION_TESTING

    def test_convert_to_json_is_possible(self):
        self.assertEqual(
            True,
            validate_slick_config(json.dumps({'is_chuck': True}))
            )

    def test_convert_to_json_is_not_possible(self):
        self.assertEqual(
            False,
            validate_slick_config("no_json")
            )



from ftw.upgrade import UpgradeStep


class AddINameFromTitleBehaviorToFtwSliderPane(UpgradeStep):
    """Add "INameFromTitle" behavior to "ftw.slider.Pane".
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from ftw.testing import IS_PLONE_5
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import login
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import TEST_USER_NAME
from zope.configuration import xmlconfig
import ftw.referencewidget.tests.widgets
import ftw.slider.tests.builders  # noqa


class SliderLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '</configure>',
            context=configurationContext)

    def setUpPloneSite(self, portal):
        login(portal, TEST_USER_NAME)
        # Install into Plone site using portal_setup
        applyProfile(portal, 'ftw.slider:default')
        if IS_PLONE_5:
            applyProfile(portal, 'plone.app.contenttypes:default')


SLIDER_TAGS_FIXTURE = SliderLayer()
SLIDER_INTEGRATION_TESTING = IntegrationTesting(
    bases=(SLIDER_TAGS_FIXTURE,),
    name="ftw.slider:integration")
SLIDER_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(SLIDER_TAGS_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.slider:functional")
